function Y = bsp15(X,flag,Parmeter);
% Himmelblau, p. 415
% flag = 1: Objective function
% flag = 2: Inequalities
% flag = 3: Equalities
% flag = 4: Gradient of objective function
% flag = 5: Gradient of inequalities
% flag = 6: Gradient of equalities
% -------------------------------------------
% Gradient of f: R_n -> R_m is (m,n)-matrix
% -------------------------------------------
switch flag
case 1
   Y = -0.5*(X(1)*X(4)-X(2)*X(3)+X(3)*X(9)-X(5)*X(9)+X(5)*X(8)-X(6)*X(7));
case 2
   g1  = 1-X(3)^2-X(4)^2;
   g2  = 1-X(9)^2;
   g3  = 1-X(5)^2-X(6)^2;
   g4  = 1-X(1)^2-(X(2)-X(9))^2;
   g5  = 1-(X(1)-X(5))^2-(X(2)-X(6))^2;
   g6  = 1-(X(1)-X(7))^2-(X(2)-X(8))^2;
   g7  = 1-(X(3)-X(5))^2-(X(4)-X(6))^2;
   g8  = 1-(X(3)-X(7))^2-(X(4)-X(8))^2;
   g9  = 1-X(7)^2-(X(8)-X(9))^2;
   g10 = X(1)*X(4)-X(2)*X(3);
   g11 = X(3)*X(9);
   g12 = -X(5)*X(9);
   g13 = X(5)*X(8)-X(6)*X(7);
   g14 = X(9);
   Y = [g1 g2 g3 g4 g5 g6 g7 g8 g9 g10 g11 g12 g13 g14]';
case 3, Y = 0;
case 4
   gradf = [X(4),-X(3),X(9)-X(2),X(1),X(8)-X(9)];
   Y = -0.5*[gradf,-X(7),-X(6),X(5),X(3)-X(5)];
case 5
   gradg = zeros(14,9);
   gradg(1,:)  = [0,0,-2*X(3),-2*X(4),0,0,0,0,0];
   gradg(2,:)  = [0,0,0,0,0,0,0,0,-2*X(9)];
   gradg(3,:)  = [0,0,0,0,-2*X(5),-2*X(6),0,0,0];
   gradg(4,:)  = 2*[-X(1),-X(2)+X(9),0,0,0,0,0,0,X(2)-X(9)];
   gradg(5,:)  = 2*[-X(1)+X(5),-X(2)+X(6),0,0,...
                 X(1)-X(5),X(2)-X(6),0,0,0];
   gradg(6,:)  = 2*[-X(1)+X(7),-X(2)+X(8),0,0,0,0,...
                -X(7)+X(1),-X(8)+X(2),0];
   gradg(7,:)  = 2*[0,0,-X(3)+X(5),-X(4)+X(6),...
                -X(5)+X(3),-X(6)+X(4),0,0,0];
   gradg(8,:)  = 2*[0,0,-X(3)+X(7),-X(4)+X(8),0,0,...
                -X(7)+X(3),-X(8)+X(4),0];
   gradg(9,:)  = 2*[0,0,0,0,0,0,-X(7),-X(8)+X(9),-X(9)+X(8)];
   gradg(10,:) = [X(4),-X(3),-X(2),X(1),0,0,0,0,0];
   gradg(11,:) = [0,0,X(9),0,0,0,0,0,X(3)];
   gradg(12,:) = [0,0,0,0,-X(9),0,0,0,-X(5)];
   gradg(13,:) = [0,0,0,0,X(8),-X(7),-X(6),X(5),0];
   gradg(14,:) = [0,0,0,0,0,0,0,0,1];
   Y = gradg;
case 6, Y = 0;
end
